@@ -10,24 +10,27 @@ import (
 	"github.com/prometheus/client_golang/prometheus"
 	"github.com/tobyxdd/hysteria/pkg/acl"
 	"github.com/tobyxdd/hysteria/pkg/obfs"
-	transport2 "github.com/tobyxdd/hysteria/pkg/transport"
+	"github.com/tobyxdd/hysteria/pkg/pmtud_fix"
+	"github.com/tobyxdd/hysteria/pkg/transport"
 	"net"
 )
 
-type AuthFunc func(addr net.Addr, auth []byte, sSend uint64, sRecv uint64) (bool, string)
+type ConnectFunc func(addr net.Addr, auth []byte, sSend uint64, sRecv uint64) (bool, string)
+type DisconnectFunc func(addr net.Addr, auth []byte, err error)
 type TCPRequestFunc func(addr net.Addr, auth []byte, reqAddr string, action acl.Action, arg string)
 type TCPErrorFunc func(addr net.Addr, auth []byte, reqAddr string, err error)
 type UDPRequestFunc func(addr net.Addr, auth []byte, sessionID uint32)
 type UDPErrorFunc func(addr net.Addr, auth []byte, sessionID uint32, err error)
 
 type Server struct {
-	transport         transport2.Transport
+	transport         *transport.ServerTransport
 	sendBPS, recvBPS  uint64
 	congestionFactory CongestionFactory
 	disableUDP        bool
 	aclEngine         *acl.Engine
 
-	authFunc       AuthFunc
+	connectFunc    ConnectFunc
+	disconnectFunc DisconnectFunc
 	tcpRequestFunc TCPRequestFunc
 	tcpErrorFunc   TCPErrorFunc
 	udpRequestFunc UDPRequestFunc
@@ -39,19 +42,16 @@ type Server struct {
 	listener quic.Listener
 }
 
-func NewServer(addr string, protocol string, tlsConfig *tls.Config, quicConfig *quic.Config, transport transport2.Transport,
+func NewServer(addr string, protocol string, tlsConfig *tls.Config, quicConfig *quic.Config, transport *transport.ServerTransport,
 	sendBPS uint64, recvBPS uint64, congestionFactory CongestionFactory, disableUDP bool, aclEngine *acl.Engine,
-	obfuscator obfs.Obfuscator, authFunc AuthFunc, tcpRequestFunc TCPRequestFunc, tcpErrorFunc TCPErrorFunc,
+	obfuscator obfs.Obfuscator, connectFunc ConnectFunc, disconnectFunc DisconnectFunc,
+	tcpRequestFunc TCPRequestFunc, tcpErrorFunc TCPErrorFunc,
 	udpRequestFunc UDPRequestFunc, udpErrorFunc UDPErrorFunc, promRegistry *prometheus.Registry) (*Server, error) {
-	pktConn, err := transport.QUICPacketConn(protocol, true, addr, "", obfuscator)
+	quicConfig.DisablePathMTUDiscovery = quicConfig.DisablePathMTUDiscovery || pmtud_fix.DisablePathMTUDiscovery
+	listener, err := transport.QUICListen(protocol, addr, tlsConfig, quicConfig, obfuscator)
 	if err != nil {
 		return nil, err
 	}
-	listener, err := quic.Listen(pktConn, tlsConfig, quicConfig)
-	if err != nil {
-		_ = pktConn.Close()
-		return nil, err
-	}
 	s := &Server{
 		listener:          listener,
 		transport:         transport,
@@ -60,7 +60,8 @@ func NewServer(addr string, protocol string, tlsConfig *tls.Config, quicConfig *
 		congestionFactory: congestionFactory,
 		disableUDP:        disableUDP,
 		aclEngine:         aclEngine,
-		authFunc:          authFunc,
+		connectFunc:       connectFunc,
+		disconnectFunc:    disconnectFunc,
 		tcpRequestFunc:    tcpRequestFunc,
 		tcpErrorFunc:      tcpErrorFunc,
 		udpRequestFunc:    udpRequestFunc,
@@ -105,7 +106,7 @@ func (s *Server) handleClient(cs quic.Session) {
 		return
 	}
 	// Handle the control stream
-	auth, ok, err := s.handleControlStream(cs, stream)
+	auth, ok, v2, err := s.handleControlStream(cs, stream)
 	if err != nil {
 		_ = cs.CloseWithError(closeErrorCodeProtocol, "protocol error")
 		return
@@ -115,33 +116,35 @@ func (s *Server) handleClient(cs quic.Session) {
 		return
 	}
 	// Start accepting streams and messages
-	sc := newServerClient(cs, s.transport, auth, s.disableUDP, s.aclEngine,
+	sc := newServerClient(v2, cs, s.transport, auth, s.disableUDP, s.aclEngine,
 		s.tcpRequestFunc, s.tcpErrorFunc, s.udpRequestFunc, s.udpErrorFunc,
 		s.upCounterVec, s.downCounterVec, s.connGaugeVec)
-	sc.Run()
+	err = sc.Run()
 	_ = cs.CloseWithError(closeErrorCodeGeneric, "")
+	s.disconnectFunc(cs.RemoteAddr(), auth, err)
 }
 
 // Auth & negotiate speed
-func (s *Server) handleControlStream(cs quic.Session, stream quic.Stream) ([]byte, bool, error) {
+func (s *Server) handleControlStream(cs quic.Session, stream quic.Stream) ([]byte, bool, bool, error) {
 	// Check version
 	vb := make([]byte, 1)
 	_, err := stream.Read(vb)
 	if err != nil {
-		return nil, false, err
+		return nil, false, false, err
 	}
-	if vb[0] != protocolVersion {
-		return nil, false, fmt.Errorf("unsupported protocol version %d, expecting %d", vb[0], protocolVersion)
+	if vb[0] != protocolVersion && vb[0] != protocolVersionV2 {
+		return nil, false, false, fmt.Errorf("unsupported protocol version %d, expecting %d/%d",
+			vb[0], protocolVersionV2, protocolVersion)
 	}
 	// Parse client hello
 	var ch clientHello
 	err = struc.Unpack(stream, &ch)
 	if err != nil {
-		return nil, false, err
+		return nil, false, false, err
 	}
 	// Speed
 	if ch.Rate.SendBPS == 0 || ch.Rate.RecvBPS == 0 {
-		return nil, false, errors.New("invalid rate from client")
+		return nil, false, false, errors.New("invalid rate from client")
 	}
 	serverSendBPS, serverRecvBPS := ch.Rate.RecvBPS, ch.Rate.SendBPS
 	if s.sendBPS > 0 && serverSendBPS > s.sendBPS {
@@ -151,7 +154,7 @@ func (s *Server) handleControlStream(cs quic.Session, stream quic.Stream) ([]byt
 		serverRecvBPS = s.recvBPS
 	}
 	// Auth
-	ok, msg := s.authFunc(cs.RemoteAddr(), ch.Auth, serverSendBPS, serverRecvBPS)
+	ok, msg := s.connectFunc(cs.RemoteAddr(), ch.Auth, serverSendBPS, serverRecvBPS)
 	// Response
 	err = struc.Pack(stream, &serverHello{
 		OK: ok,
@@ -162,11 +165,11 @@ func (s *Server) handleControlStream(cs quic.Session, stream quic.Stream) ([]byt
 		Message: msg,
 	})
 	if err != nil {
-		return nil, false, err
+		return nil, false, false, err
 	}
 	// Set the congestion accordingly
 	if ok && s.congestionFactory != nil {
 		cs.SetCongestionControl(s.congestionFactory(serverSendBPS))
 	}
-	return ch.Auth, ok, nil
+	return ch.Auth, ok, vb[0] == protocolVersionV2, nil
 }
